package ru.octol1ttle.scamstopper

import dev.kord.common.entity.Snowflake
import dev.kord.core.behavior.edit
import dev.kord.core.behavior.getChannelOf
import dev.kord.core.entity.channel.TextChannel
import dev.kord.core.event.message.MessageCreateEvent
import kotlinx.datetime.Clock
import mu.KLogger
import mu.KotlinLogging
import kotlin.time.Duration.Companion.days

class MessageCreateListener {
    private val logger: KLogger = KotlinLogging.logger { }
    private val messageMap = HashMap<Snowflake, FlaggedMessage>()

    suspend fun execute(event: MessageCreateEvent) {
        if (event.member == null) {
            logger.debug("Skipped message processing because 'member' was null")
            return
        }

        if (!event.message.content.contains("discord.")
            && !event.message.content.contains("http://")
            && !event.message.content.contains("https://")
        ) {
            logger.debug("Skipped message processing because no URLs or Discord invite links were found")
            return
        }

        val key: Snowflake = event.message.author!!.id

        val flagged = messageMap[key]
        if (flagged?.content.equals(event.message.content, true)) {
            val guild = event.message.getGuild()
            val username = event.message.author?.username
            logger.info("Matched duplicate message from $username, will attempt to delete it and time out")

            try {
                event.message.delete("Flagged by Scam Stopper")
                guild.getChannelOf<TextChannel>(flagged!!.channelId).deleteMessage(flagged.id, "Flagged by Scam Stopper")
            } catch (e: Exception) {
                logger.warn("Error deleting messages", e)
            }

            try {
                event.member!!.edit {
                    communicationDisabledUntil = Clock.System.now().plus(7.days)
                    reason = "Flagged by Scam Stopper"
                }
            } catch (e: Exception) {
                logger.warn("Error timing out user", e)
            }

            logger.info("Timed out $username for 7 days and deleted their messages, see matched message below")
            logger.info(event.message.content)

            configs!![event.message.data.guildId.value]?.logChannel?.let {
                guild.getChannelOf<TextChannel>(it).createMessage("Flagged and deleted duplicate messages from $username, timed them out for 7 days")
            }

            return
        }

        logger.debug("No duplicates were found")
        messageMap[key] = FlaggedMessage(event.message.channelId, event.message.id, event.message.content)
    }
}